# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import director_pb2 as director__pb2

GRPC_GENERATED_VERSION = '1.78.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in director_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class DirectorServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Review = channel.unary_unary(
                '/director.DirectorService/Review',
                request_serializer=director__pb2.ReviewRequest.SerializeToString,
                response_deserializer=director__pb2.ReviewResponse.FromString,
                _registered_method=True)
        self.Process = channel.unary_unary(
                '/director.DirectorService/Process',
                request_serializer=director__pb2.ProcessRequest.SerializeToString,
                response_deserializer=director__pb2.ProcessResponse.FromString,
                _registered_method=True)
        self.ReviewBatch = channel.unary_unary(
                '/director.DirectorService/ReviewBatch',
                request_serializer=director__pb2.BatchReviewRequest.SerializeToString,
                response_deserializer=director__pb2.BatchReviewResponse.FromString,
                _registered_method=True)
        self.StreamTokens = channel.unary_stream(
                '/director.DirectorService/StreamTokens',
                request_serializer=director__pb2.StreamRequest.SerializeToString,
                response_deserializer=director__pb2.TokenEvent.FromString,
                _registered_method=True)


class DirectorServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Review(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Process(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReviewBatch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamTokens(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DirectorServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Review': grpc.unary_unary_rpc_method_handler(
                    servicer.Review,
                    request_deserializer=director__pb2.ReviewRequest.FromString,
                    response_serializer=director__pb2.ReviewResponse.SerializeToString,
            ),
            'Process': grpc.unary_unary_rpc_method_handler(
                    servicer.Process,
                    request_deserializer=director__pb2.ProcessRequest.FromString,
                    response_serializer=director__pb2.ProcessResponse.SerializeToString,
            ),
            'ReviewBatch': grpc.unary_unary_rpc_method_handler(
                    servicer.ReviewBatch,
                    request_deserializer=director__pb2.BatchReviewRequest.FromString,
                    response_serializer=director__pb2.BatchReviewResponse.SerializeToString,
            ),
            'StreamTokens': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamTokens,
                    request_deserializer=director__pb2.StreamRequest.FromString,
                    response_serializer=director__pb2.TokenEvent.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'director.DirectorService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('director.DirectorService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class DirectorService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Review(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/director.DirectorService/Review',
            director__pb2.ReviewRequest.SerializeToString,
            director__pb2.ReviewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Process(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/director.DirectorService/Process',
            director__pb2.ProcessRequest.SerializeToString,
            director__pb2.ProcessResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReviewBatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/director.DirectorService/ReviewBatch',
            director__pb2.BatchReviewRequest.SerializeToString,
            director__pb2.BatchReviewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StreamTokens(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/director.DirectorService/StreamTokens',
            director__pb2.StreamRequest.SerializeToString,
            director__pb2.TokenEvent.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)
