# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: director.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'director.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64irector.proto\x12\x08\x64irector\"E\n\rReviewRequest\x12\x0e\n\x06prompt\x18\x01 \x01(\t\x12\x10\n\x08response\x18\x02 \x01(\t\x12\x12\n\nsession_id\x18\x03 \x01(\t\"l\n\x0eReviewResponse\x12\x10\n\x08\x61pproved\x18\x01 \x01(\x08\x12\x11\n\tcoherence\x18\x02 \x01(\x01\x12\x11\n\th_logical\x18\x03 \x01(\x01\x12\x11\n\th_factual\x18\x04 \x01(\x01\x12\x0f\n\x07warning\x18\x05 \x01(\x08\" \n\x0eProcessRequest\x12\x0e\n\x06prompt\x18\x01 \x01(\t\"\x8a\x01\n\x0fProcessResponse\x12\x0e\n\x06output\x18\x01 \x01(\t\x12\x11\n\tcoherence\x18\x02 \x01(\x01\x12\x0e\n\x06halted\x18\x03 \x01(\x08\x12\x1c\n\x14\x63\x61ndidates_evaluated\x18\x04 \x01(\x05\x12\x0f\n\x07warning\x18\x05 \x01(\x08\x12\x15\n\rfallback_used\x18\x06 \x01(\x08\"?\n\x12\x42\x61tchReviewRequest\x12)\n\x08requests\x18\x01 \x03(\x0b\x32\x17.director.ReviewRequest\"B\n\x13\x42\x61tchReviewResponse\x12+\n\tresponses\x18\x01 \x03(\x0b\x32\x18.director.ReviewResponse\"\x1f\n\rStreamRequest\x12\x0e\n\x06prompt\x18\x01 \x01(\t\"b\n\nTokenEvent\x12\r\n\x05token\x18\x01 \x01(\t\x12\x11\n\tcoherence\x18\x02 \x01(\x01\x12\r\n\x05index\x18\x03 \x01(\x05\x12\x0e\n\x06halted\x18\x04 \x01(\x08\x12\x13\n\x0bhalt_reason\x18\x05 \x01(\t2\x9b\x02\n\x0f\x44irectorService\x12;\n\x06Review\x12\x17.director.ReviewRequest\x1a\x18.director.ReviewResponse\x12>\n\x07Process\x12\x18.director.ProcessRequest\x1a\x19.director.ProcessResponse\x12J\n\x0bReviewBatch\x12\x1c.director.BatchReviewRequest\x1a\x1d.director.BatchReviewResponse\x12?\n\x0cStreamTokens\x12\x17.director.StreamRequest\x1a\x14.director.TokenEvent0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'director_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_REVIEWREQUEST']._serialized_start=28
  _globals['_REVIEWREQUEST']._serialized_end=97
  _globals['_REVIEWRESPONSE']._serialized_start=99
  _globals['_REVIEWRESPONSE']._serialized_end=207
  _globals['_PROCESSREQUEST']._serialized_start=209
  _globals['_PROCESSREQUEST']._serialized_end=241
  _globals['_PROCESSRESPONSE']._serialized_start=244
  _globals['_PROCESSRESPONSE']._serialized_end=382
  _globals['_BATCHREVIEWREQUEST']._serialized_start=384
  _globals['_BATCHREVIEWREQUEST']._serialized_end=447
  _globals['_BATCHREVIEWRESPONSE']._serialized_start=449
  _globals['_BATCHREVIEWRESPONSE']._serialized_end=515
  _globals['_STREAMREQUEST']._serialized_start=517
  _globals['_STREAMREQUEST']._serialized_end=548
  _globals['_TOKENEVENT']._serialized_start=550
  _globals['_TOKENEVENT']._serialized_end=648
  _globals['_DIRECTORSERVICE']._serialized_start=651
  _globals['_DIRECTORSERVICE']._serialized_end=934
# @@protoc_insertion_point(module_scope)
